/**
 * @namespace UIScheme
 */
declare namespace UIScheme
{
    class Element
    {
        constructor(selector: string);

        /**
         * Sets one or few styles of the node.
         * If style argument is string only one style will be set to value.
         * If style argument is object styles will be set from its properties
         * and value will be omitted.
         */
        css(style: string, value: string);

        /**
         * Sets or gets attribute of the node.
         * To get attribute value must be omitted.
         */
         attr(name: string, value: string);

        /**
         * Returns node of the element.
         */
         node(): HTMLElement;

        /**
         * Adds event listener.
         * Named listeners can be used:
         * chat.on('message -> tray', msg => console.log(msg))
         */
        on(eventName: string, handler: any);

        /**
         * Removes event listener.
         */
        off(eventName: string, handler: any);

        /**
         * Fires event with given name.
         */
        trigger(eventName: string, ...data: any)
    }

    class Scheme
    {
        /**
         * @param {string} selector Common css selector.
         * @param {Object} data Nested scheme.
         */
        constructor(selector: string, data ?: any);

        /**
         * Builds scheme and returns root element.
         *
         * @param {Scheme} scheme
         * @param {string} [namespace]
         */
        static build(scheme : Scheme, namespace ?: string): Element
    }


    class UI {
        constructor(...args: any);


        rootElement: Element;

        /**
         * Allows to initialize additional parameters before rendering.
         * This method may be overridden in sub-class.
         */
        init(...params: any);

        /**
         * Creates scheme of the UI.
         * The good reason is to keep all structure inside one container (usually named "wrap").
         *
         * @return {Scheme}
         */
        createScheme(): Scheme;

        /**
         * Overrides parameters that was defined for rendering.
         * This method called before creating any nodes but after initialization of parameters.
         */
        onBeforeRender(params: Object);

        /**
         * Handles rendering of the UI.
         * This method called just after instance creation.
         */
        onRender(params: Object);

        /**
         * Renders UI with given parameters.
         */
        render(params: Object);

        /**
         * Override this method to change target in which UI will be rendered to.
         * To change target just return another target.
         *
         * @param {string|Element} target
         */
        onBeforeAppend(target: any);

        /**
         * Appends UI to the given target.
         * If target is string it will be considered as CSS selector.
         *
         * @param {string|Element|UI} target
         */
        appendTo(target: any);

        /**
         * This method will be called just after appending.
         *
         * @param {string|Element} target
         */
        onAppend(target: any);

        /**
         * Modifies or overrides default loading logic.
         * To override loading prevent default of the given event.
         * This method should modify given data object.
         */
        onLoad(data: Object, event: Event);

        /**
         * Modifies or overrides default gathering logic.
         * To override gathering prevent default of the given event.
         */
        onGather(data: Object, event: Event);

        /**
         * Returns styles of the UI.
         * This method must be overridden in the child class.
         */
        createStyles(): Styles;

        /**
         * Defines initial parameters of the UI.
         * Can be overridden by sub-class.
         * Can returns any static object because all properties will be copied in the final params.
         * IMPORTANT! Nested objects will be copied as references.
         *
         * @return {Object|null}
         */
        defaultParams(): any;

        /**
         * Adds event listener.
         * Named listeners can be used:
         * chat.on('message -> tray', msg => console.log(msg))
         */
        on(eventName: string, handler: any);

        /**
         * Removes event listener.
         */
        off(eventName: string, handler: any);

        /**
         * Fires event with given name.
         */
        trigger(eventName: string, ...data: any)
    }



    class Styles {
        constructor(data: Object);

        renderFor(ui: UI, enforce: boolean);


        static renderRules(target, selector, rules);
    }


    class EventsChannel {
        constructor(context: any);

        on(eventName: string, handler: any);

        off(eventName: string);

        trigger(eventName: string, ...params: any);

        triggerDirect(eventName: string, ...params: any);

        static get(name: string, context: any);
    }


    class Localization
    {
        constructor(data: Object);

        switchLocaleTo(locale: string);

        /**
         * Adds event listener.
         * Can be used named listeners:
         * chat.on('newMessage -> tray', function() { // ... });
         *
         * @param {string} eventName
         * @param {function} handler
         */
        on(eventName: string, handler: any);

        /**
         * Removes event listener.
         *
         * @param {string} eventName
         * @param {function} [handler]
         */
        off(eventName: string, handler: any);

        /**
         * Fires event with given name.
         */
        trigger(eventName: string, ...data: any)
    }
}

declare module "ui-scheme" {
    export = UIScheme;
}